@@ -500,7 +500,7 @@ update_cache (EBookBackendExchange *be)
 	EContact *contact;
 	const char *cache_file_name;
 	time_t mod_time;
-	char time_string[25];
+	char time_string[50]; /* Donno what could be right. 50 should be safe for sure.*/
 	const struct tm *tm;
 	struct stat buf;
 
@@ -510,7 +510,7 @@ update_cache (EBookBackendExchange *be)
 	stat (cache_file_name, &buf);
 	mod_time = buf.st_mtime;
 	tm = gmtime (&mod_time);
-	strftime (time_string, 100, "%Y-%m-%dT%H:%M:%SZ", tm);
+	strftime (time_string, 50, "%Y-%m-%dT%H:%M:%SZ", tm);
 
 	/* build hash table from storage file */
 
@@ -1273,7 +1273,7 @@ do_put (EBookBackendExchange *be, EDataBook *book,
 	body = build_message (hier->owner_name, hier->owner_email,
 			      subject, note, photo);
 	status = e2k_context_put (be->priv->ctx, NULL, uri, "message/rfc822",
-				  body->data, body->len, NULL);
+				  (char *) body->data, body->len, NULL);
 	g_byte_array_free (body, TRUE);
 
 	return status;
@@ -1415,7 +1415,7 @@ e_book_backend_exchange_modify_contact (EBookBackendSync  *backend,
 			/* Do the PUT request if we need to. */
 			char *old_note, *new_note;
 			EContactPhoto *old_photo, *new_photo;
-			gboolean changed;
+			gboolean changed = FALSE;
 
 			old_note = e_contact_get (old_contact, E_CONTACT_NOTE);
 			old_photo = e_contact_get (old_contact, E_CONTACT_PHOTO);
@@ -1441,8 +1441,6 @@ e_book_backend_exchange_modify_contact (EBookBackendSync  *backend,
 					  old_photo->type == E_CONTACT_PHOTO_TYPE_URI) {
 					changed = !strcmp (old_photo->data.uri, new_photo->data.uri);
 				}
-				else
-					changed = FALSE;
 			}
 
 			if (changed) {
@@ -2110,7 +2108,10 @@ e_book_backend_exchange_get_contact (EBookBackendSync  *backend,
 	EBookBackendExchange *be = E_BOOK_BACKEND_EXCHANGE (backend);
 	EBookBackendExchangePrivate *bepriv = be->priv;
 	EContact *contact;
-
+	E2kResult *results=NULL;
+	E2kHTTPStatus status;
+	int nresults = 0;
+	
 	d(printf("ebbe_get_contact(%p, %p, %s)\n", backend, book, id));
 
 	be = E_BOOK_BACKEND_EXCHANGE (e_data_book_get_backend (book));
@@ -2132,7 +2133,55 @@ e_book_backend_exchange_get_contact (EBookBackendSync  *backend,
 		}
 		
 	case GNOME_Evolution_Addressbook_MODE_REMOTE:
-		/* XXX finish this */
+
+		if (bepriv->marked_for_offline && e_book_backend_cache_is_populated (bepriv->cache)) {
+			contact = e_book_backend_cache_get_contact (bepriv->cache,
+								    id);
+			if (contact) {
+				*vcard =  e_vcard_to_string (E_VCARD (contact), 
+							     EVC_FORMAT_VCARD_30);
+				g_object_unref (contact);
+				return GNOME_Evolution_Addressbook_Success;
+			}
+			else {
+				*vcard = g_strdup ("");
+				return GNOME_Evolution_Addressbook_ContactNotFound;
+			}
+
+		} else {
+			E2kUri *uri;
+
+			/* Dont allow any non uri to go GAL UID is not a uri */
+			uri = e2k_uri_new (id);
+			if (!uri->protocol ||  !*uri->protocol) {
+				e2k_uri_free (uri);
+				*vcard = g_strdup ("");
+				return GNOME_Evolution_Addressbook_ContactNotFound;
+			}
+			e2k_uri_free (uri);
+
+			status = e2k_context_propfind (bepriv->ctx, NULL, id,
+						       field_names, n_field_names,
+						       &results, &nresults);
+		
+			if (status == E2K_HTTP_CANCELLED)
+				return GNOME_Evolution_Addressbook_OtherError;
+
+			if (status == E2K_HTTP_MULTI_STATUS && nresults > 0) {
+				contact = e_contact_from_props (be, &results[0]);
+				*vcard =  e_vcard_to_string (E_VCARD (contact), 
+							     EVC_FORMAT_VCARD_30);
+				g_object_unref (contact);
+				e2k_results_free (results, nresults);
+				
+				return GNOME_Evolution_Addressbook_Success;
+			
+			} else {
+				*vcard = g_strdup ("");
+				
+				return GNOME_Evolution_Addressbook_ContactNotFound;			
+			}
+		}
 
 	default:
 		break;
